;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;This script used to calculate and plot the zonal mean variables from CAM;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;The terms zonal and meridional are used to describe directions on a globe;;;;;;;
;;;;;;;Zonal means "along a latitudinal circle" or "in the west–east direction";;;;;;;;
;;;;;;;while meridional means "along a longitudinal circle" (a.k.a. meridian) ;;;;;;;;;
;;;;;;;or "in the north–south direction" [2];;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl"

begin

  ModelOutRoot = "./output/"
  Casdir       = (/"CLIM30","CLIM30ML"/) 
  Groups       = Casdir
  ngrps        = dimsizes(Groups)
  yst          = 1979
  yed          = 2014
  regions      = (/"Polar","MidLat","Tropics","Global"/)
  nregs        = dimsizes(regions)
  regions@long_name = "1 - Polar region, 90S-60S, 60N-90N; " + \
                      "2 - Mid-latitude, 60S-30S, 30N-60N; " + \
                      "3 - Tropical zone, 20S-20N; " + \
                      "4 - Global"
  
 ;;;;;;;begin to work with the simulations;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  xlabnam = (/"PSL",  "U850", "U500", "U200", \
              "T850", "T500", "T200", \
              "Q850", "Q500", "Q200"/)
  varnam  = (/"PSL",  "U850", "U500", "U200", \
              "T850", "T500", "T200", \
              "Q850", "Q500", "Q200"/)
  nvars   = dimsizes(varnam)

 ovarnam                 = (/xlabnam/)
 ovarnam!0               = "nvars"
 ovarnam&nvars           = ispan(1,nvars,1)
 ovarnam&nvars@long_name = "number of variables"

 ;;read and calculate the mean and rmsd;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

 do ir = 0, nregs-1

  ;;open netcdf file to save the data for plots;;;;;
  setfileoption("nc", "Format",  "NetCDF4")
  out_file_name = "Bias_and_RMS_E3SM_vs_OBS_"+regions(ir)+"_"+yst+"-"+yed+".nc"
  system("rm " + out_file_name)
  fout = addfile(out_file_name,"c")
  fout->Evaluated_variables=ovarnam

  do igp = 0,ngrps-1

    fname  = ModelOutRoot+"/"+Groups(igp)+"_"+regions(ir)+"_clim_output_"+yst+"01-"+yed+"12.txt"
    data   = asciiread(fname,-1,"string")
    vstr   = str_get_field(data(1::), 1," ")
    vunt   = str_get_field(data(1::), 2," ")
    mobs   = stringtofloat(str_get_field(data(1::), 4," "))
    rmso   = stringtofloat(str_get_field(data(1::), 5," "))
    mmod   = stringtofloat(str_get_field(data(1::), 6," "))
    rmsd   = stringtofloat(str_get_field(data(1::), 9," "))
    merr   = new((/nvars/),float)
    mrms   = new((/nvars/),float)

    do i = 0,nvars-1,1
      do j = 0,dimsizes(vstr)-1,1
        if (str_squeeze(vstr(j)).eq.varnam(i)) then
          print(vstr(j) + " " + varnam(i))
          merr(i) = (mmod(j)-mobs(j))/mobs(j)
          mrms(i) = rmsd(j)/rmso(j)
        end if
      end do
    end do
   ;;;;open a netcdf file and save the data for plot;;;;;
   vbias = merr
   vrmse = mrms

   vbias!0     = "nvars"
   vbias&nvars = ispan(1,nvars,1)
   vbias&nvars@long_name = "number of variables"
  
   vrmse!0     = "nvars"
   vrmse&nvars = ispan(1,nvars,1)
   vrmse&nvars@long_name = "number of variables"
 
   ;;;save the data to netcdf file;;;;
   vnmbias = "Bias_"+Groups(igp)
   vnmrmse = "RMSE_"+Groups(igp)
   fout->$vnmbias$=vbias
   fout->$vnmrmse$=vrmse

   delete([/vbias,vrmse/])
 end do ; end igp loop

end do 

end

