load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl"

undef("nice_string")
  
function nice_string(var)

begin

  string_len = sizeof(stringtochar(var)) - 1

  if (string_len .lt. 25) then
      ilen = 16
      do it = 0, ilen - string_len
         var = var + " "
      end do
  end if

  return(var)

end

;;;;;;;;;;;;;;;;
; main program ;
;;;;;;;;;;;;;;;;

begin

  model     = getenv("MODEL_NAME")
  obs       = getenv("OBS_NAME")
  Input     = getenv("INPUT")
  var       = getenv("VAR")
  vlev      = getenv("VLEV")
  yst       = toint(getenv("yst"))
  yed       = toint(getenv("yed"))

  region_lobnd          = (/-90, -60, -20, -90/)
  region_hibnd          = (/ 90,  60,  20,  90/)
  region_name           = (/1, 2, 3, 4/)
  region_name@long_name = "1 - Polar region, 90S-60S, 60N-90N; " + \
                          "2 - Mid-latitude, 60S-30S, 30N-60N; " + \
                          "3 - Tropical zone, 20S-20N; " + \
                          "4 - Global"
  regid                 = (/0,1,2,3/)
  regions               = (/"Polar","MidLat","Tropics","Global"/)
  nreg                  = dimsizes(regid)

  modeldir  = Input+"/"+model+"/monthly/"
  flm       = systemfunc("ls "+modeldir+"*"+vlev+"*.nc")
  obsdir    = Input+"/"+obs+"/monthly/"
  flo       = systemfunc("ls "+obsdir+"*"+vlev+"*.nc")
  
  fm        = addfiles(flm, "r")
  modtime   = cd_calendar(fm[:]->time, -1)
  modind    = ind(modtime.ge.yst.and.modtime.le.yed)
  vmod      = fm[:]->$var$(modind,:,:)

  fo        = addfiles(flo, "r")
  obstime   = cd_calendar(fo[:]->time, -1)
  obsind    = ind(obstime.ge.yst.and.obstime.le.yed)
  if (var.eq."TUQ") then 
    if(isfilevar(fo[0],var)) then
      vobs  = fo[:]->$var$(obsind,:,:)
    else if(isfilevar(fo[0],"VIWVE")) then 
      vobs  = fo[:]->VIWVE(obsind,:,:)
    else 
      vobs  = vmod 
      vobs  = -99999
    end if 
    end if 
  else if (var.eq."TVQ") then
    if(isfilevar(fo[0],var)) then
      vobs  = fo[:]->$var$(obsind,:,:)
    else if(isfilevar(fo[0],"VIWVN")) then
      vobs  = fo[:]->VIWVN(obsind,:,:)
    else
      vobs  = vmod
      vobs  = -99999
    end if
    end if
  else
    vobs  = fo[:]->$var$(obsind,:,:)
  end if 
  end if 

  vmod@_FillValue = -99999
  vobs@_FillValue = -99999

  if (var .eq. "PSL") then
    vmod = vmod / 100.0
    vobs = vobs / 100.0
    vmod@units = "hPa"
    vobs@units = "hPa"
  end if

  if(isStrSubset(var,"Q").and.(var.ne."TUQ".or.var.ne."TVQ")) then 
    vmod = vmod * 1000.0
    vobs = vobs * 1000.0
    vmod@units = "g/kg"
    vobs@units = "g/kg"   
  end if 

 ;mask fill values 
  if (.not.all(ismissing(vobs))) then 
    vmod = where (ismissing(vobs), vmod@_FillValue, vmod)
    vobs = where (ismissing(vmod), vobs@_FillValue, vobs)
  end if 

 ;loop each region
  do ii = 0,nreg-1,1
   regnm = regions(ii)
   vtmp0 = vmod
   vtmp1 = vobs
   if(regnm.eq."Polar") then
     vtmp0(:,{-60:60},:) = vmod@_FillValue
     vtmp1(:,{-60:60},:) = vobs@_FillValue
   end if
   if(regnm.eq."MidLat") then
     vtmp0(:,{-90:-60},:) = vmod@_FillValue
     vtmp1(:,{-90:-60},:) = vobs@_FillValue
     vtmp0(:,{60:90},:)   = vmod@_FillValue
     vtmp1(:,{60:90},:)   = vobs@_FillValue
     vtmp0(:,{-30:30},:)  = vmod@_FillValue
     vtmp1(:,{-30:30},:)  = vobs@_FillValue
   end if
   if(regnm.eq."Tropics") then
     vtmp0(:,{-90:-20},:) = vmod@_FillValue
     vtmp1(:,{-90:-20},:) = vobs@_FillValue
     vtmp0(:,{20:90},:)   = vmod@_FillValue
     vtmp1(:,{20:90},:)   = vobs@_FillValue
   end if
   if(regnm.eq."Global") then
     vtmp0(:,{-90:-75},:) = vmod@_FillValue
     vtmp1(:,{-90:-75},:) = vobs@_FillValue
     vtmp0(:,{75:90},:)   = vmod@_FillValue
     vtmp1(:,{75:90},:)   = vobs@_FillValue
   end if

   ;calculate time average
   vmmod = dim_avg_n_Wrap(vtmp0,0)
   vmobs = dim_avg_n_Wrap(vtmp1,0)

   vmdif = vmmod
   vmdif = vmmod - vmobs

   vm1d  = ndtooned(vmmod)
   vo1d  = ndtooned(vmobs)
   vm1d@_FillValue = vmod@_FillValue
   vo1d@_FillValue = vobs@_FillValue
   varunt = vmod@units

  ;process metrics
   metrics    = (/"Variable", "Unit", "Observation", "Mean(obs)", "RMS(obs)", "Mean(model)", "RMS(model)", \
                  "Bias(m-o)", "RMSE(m-o)", "PCORR(m-o)"/)

   nmetric    = dimsizes(metrics)
   vmetric    = new(nmetric-2,double) 

   lat        = fm[0]->lat
   rad        = 4.0*atan(1.0)/180.0
   clat       = tofloat(cos(lat*rad))
   
   vmetric(0) = wgt_areaave(vmobs, clat, 1.0, 0)
   vmetric(1) = dim_stddev(vo1d)
   vmetric(2) = wgt_areaave(vmmod, clat, 1.0, 0)
   vmetric(3) = dim_stddev(vm1d)
   vmetric(4) = wgt_areaave(vmdif, clat, 1.0, 0)
   vmetric(5) = wgt_arearmse(vmmod, vmobs, clat, 1.0, 0)
   vmetric(6) = pattern_cor(vmmod, vmobs,clat, 1)

   if (all(ismissing(vmobs)))then 
     vmetric(0) = -999.0
     vmetric(1) = -999.0
     vmetric(2) = -999.0
     vmetric(3) = -999.0
     vmetric(4) = -999.0
     vmetric(5) = -999.0
     vmetric(6) = -999.0
   end if 

   delete([/clat,vtmp0,vtmp1,vmmod,vmobs,vmdif,vm1d,vo1d/])

   ;;;;;;;;;;
   ; Output ;
   ;;;;;;;;;;

    header = ""
    str    = ""
    do i = 0, nmetric-1
     header = header + str_insert(metrics(i)," ", 12)
     if(i.eq.0)then
       tmp    = var
       str    = str + str_insert(tmp," ",10)
     else if(i.eq.1)then
       tmp    = "("+varunt+")"
       str    = str + str_insert(tmp," ",10)
     else if (i.eq.2)then
       str    = str + str_insert(obs," ",10)
     else
       str    = str + sprintf("%10.3f",vmetric(i-3))
     end if
     end if
     end if
    end do

   fname   = "./output/"+model+"_"+regnm+"_clim_output_"+yst+"-"+yed+".txt"
   if (fileexists(fname)) then
     write_table(fname, "a", [/str/], "%s")
   else
     asciiwrite(fname, header)
     write_table(fname, "a", [/str/], "%s")
   end if 

 end do 
 delete([/modtime,modind,flm,fm,vmod,vobs/])

end
